@@ -58,12 +58,13 @@ import os
 import sys
 import re
 
-from mercurial import util, sslutil
+from mercurial import util, sslutil, error
 from mercurial.i18n import _
 from mercurial.url import passwordmgr
 from mercurial import mail
 from mercurial.mail import SMTPS, STARTTLS
 from mercurial import encoding
+from mercurial import ui as uimod
 
 # pylint: disable=invalid-name, line-too-long, protected-access, too-many-arguments
 
@@ -95,12 +96,13 @@ def import_meu():
         try:
             import mercurial_extension_utils
         except ImportError:
-            raise util.Abort(_("""Can not import mercurial_extension_utils.
+            raise error.Abort(_("""Can not import mercurial_extension_utils.
 Please install this module in Python path.
 See Installation chapter in https://bitbucket.org/Mekk/mercurial_keyring/ for details
 (and for info about TortoiseHG on Windows, or other bundled Python)."""))
     return mercurial_extension_utils
 
+
 meu = import_meu()
 monkeypatch_method = meu.monkeypatch_method
 
@@ -140,6 +142,7 @@ def import_keyring():
         meu.disable_logging("keyring")
     return mod
 
+
 #################################################################
 # Actual implementation
 #################################################################
@@ -203,7 +206,12 @@ class PasswordStore(object):
     def _read_password_from_keyring(pwdkey):
         """Physically read from keyring"""
         keyring = import_keyring()
-        password = keyring.get_password(KEYRING_SERVICE, pwdkey)
+        try:
+            password = keyring.get_password(KEYRING_SERVICE, pwdkey)
+        except Exception as err:
+            ui = uimod.ui()
+            ui.warn(_("keyring: keyring backend doesn't seem to work, password can not be restored. Falling back to prompts. Error details: %s\n") % str(err))
+            return ''
         # Reverse recoding from next routine
         if isinstance(password, unicode):
             return encoding.tolocal(password.encode('utf-8'))
@@ -216,8 +224,13 @@ class PasswordStore(object):
         # keyring in general expects unicode.
         # Mercurial provides "local" encoding. See #33
         password = encoding.fromlocal(password).decode('utf-8')
-        keyring.set_password(
-            KEYRING_SERVICE, pwdkey, password)
+        try:
+            keyring.set_password(
+                KEYRING_SERVICE, pwdkey, password)
+        except Exception as err:
+            ui = uimod.ui()
+            ui.warn(_("keyring: keyring backend doesn't seem to work, password was not saved. Error details: %s\n") % str(err))
+
 
 password_store = PasswordStore()
 
@@ -250,6 +263,7 @@ class PwdCache(object):
 
 _re_http_url = re.compile(r'^https?://')
 
+
 def is_http_path(url):
     return bool(_re_http_url.search(url))
 
@@ -328,7 +342,7 @@ class HTTPPasswordHandler(object):
         auth_user, auth_pwd, keyring_url = self.get_url_config(
             ui, parsed_url, actual_user)
         if auth_user and actual_user and (actual_user != auth_user):
-            raise util.Abort(_('keyring: username for %s specified both in repository path (%s) and in .hg/hgrc/[auth] (%s). Please, leave only one of those' % (base_url, actual_user, auth_user)))
+            raise error.Abort(_('keyring: username for %s specified both in repository path (%s) and in .hg/hgrc/[auth] (%s). Please, leave only one of those' % (base_url, actual_user, auth_user)))
         if auth_user and auth_pwd:
             return auth_user, auth_pwd, self.SRC_CFGAUTH, keyring_url
 
@@ -356,7 +370,7 @@ class HTTPPasswordHandler(object):
     def prompt_interactively(ui, user, realm, url):
         """Actual interactive prompt"""
         if not ui.interactive():
-            raise util.Abort(_('keyring: http authorization required but program used in non-interactive mode'))
+            raise error.Abort(_('keyring: http authorization required but program used in non-interactive mode'))
 
         if not user:
             ui.status(_("keyring: username not specified in hgrc (or in url). Password will not be saved.\n"))
@@ -607,38 +621,39 @@ def try_smtp_login(ui, smtp_obj, username, password):
             ui.status(_("SMTP login failed: %s\n\n") % inst.smtp_error)
             return False
         else:
-            raise util.Abort(inst)
+            raise error.Abort(inst)
 
 
 def keyring_supported_smtp(ui, username):
     """
-    keyring-integrated replacement for mercurial.mail._smtp
-    Used only when configuration file contains username, but
-    does not contain the password.
+    keyring-integrated replacement for mercurial.mail._smtp Used only
+    when configuration file contains username, but does not contain
+    the password.
 
     Most of the routine below is copied as-is from
-    mercurial.mail._smtp. The only changed part is
-    marked with # >>>>> and # <<<<< markers
+    mercurial.mail._smtp.  The critical changed part is marked with #
+    >>>>> and # <<<<< markers, there are also some fixes which make
+    the code working on various Mercurials (like parsebool import).
     """
+    
+    try:
+        from mercurial.utils.stringutil import parsebool
+    except ImportError:
+        from mercurial.utils import parsebool
+
     local_hostname = ui.config('smtp', 'local_hostname')
     tls = ui.config('smtp', 'tls', 'none')
     # backward compatible: when tls = true, we use starttls.
-    starttls = tls == 'starttls' or util.parsebool(tls)
+    starttls = tls == 'starttls' or parsebool(tls)
     smtps = tls == 'smtps'
     if (starttls or smtps) and not util.safehasattr(socket, 'ssl'):
-        raise util.Abort(_("can't use TLS: Python SSL support not installed"))
+        raise error.Abort(_("can't use TLS: Python SSL support not installed"))
     mailhost = ui.config('smtp', 'host')
     if not mailhost:
-        raise util.Abort(_('smtp.host not configured - cannot send mail'))
-    verifycert = ui.config('smtp', 'verifycert', 'strict')
-    if verifycert not in ['strict', 'loose']:
-        if util.parsebool(verifycert) is not False:
-            raise util.Abort(_('invalid smtp.verifycert configuration: %s')
-                             % (verifycert))
-        verifycert = False
+        raise error.Abort(_('smtp.host not configured - cannot send mail'))
     if getattr(sslutil, 'sslkwargs', None) is None:
         sslkwargs = None
-    elif (starttls or smtps) and verifycert:
+    elif starttls or smtps:
         sslkwargs = sslutil.sslkwargs(ui, mailhost)
     else:
         sslkwargs = {}
@@ -674,12 +689,10 @@ def keyring_supported_smtp(ui, username):
         s.ehlo()
         s.starttls()
         s.ehlo()
-    if (starttls or smtps) and verifycert:
-        ui.note(_('(verifying remote certificate)\n'))
+    if starttls or smtps:
         if getattr(sslutil, 'validatesocket', None):
+            ui.note(_('(verifying remote certificate)\n'))
             sslutil.validatesocket(s.sock)
-        else:
-            validator(ui, mailhost)(s.sock, verifycert == 'strict')
 
     # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     stored = password = password_store.get_smtp_password(
@@ -699,9 +712,9 @@ def keyring_supported_smtp(ui, username):
             return s.sendmail(sender, recipients, msg)
         except smtplib.SMTPRecipientsRefused, inst:
             recipients = [r[1] for r in inst.recipients.values()]
-            raise util.Abort('\n' + '\n'.join(recipients))
+            raise error.Abort('\n' + '\n'.join(recipients))
         except smtplib.SMTPException, inst:
-            raise util.Abort(inst)
+            raise error.Abort(inst)
 
     return send
 
